<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/config.php');
require_once(dirname(__FILE__).'/php.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Runtime class handles the checks for accessing a
// restricted post, saving restrictions for a post and modifying the RSS feed
// ----------------------------------------------------------------------------

class DiscloseSecret_Runtime
{
	var $m_cache;
	var $m_save_as_private = false;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Runtime(&$cache, $activation_status)
	{
		$this->m_cache = $cache;

		if ($activation_status != DiscloseSecret_STATUS_OKAY)
			return;

		add_filter('content_save_pre', array(&$this, 'filter_content_save_pre'));
		add_filter('status_save_pre', array(&$this, 'filter_status_save_pre'));
		add_filter('save_post', array(&$this, 'filter_post_save'));

		add_filter('posts_join', array(&$this, 'filter_posts_join'), 10, 2);
		add_filter('posts_where', array(&$this, 'filter_posts_where'));
		add_filter('user_has_cap', array(&$this, 'filter_user_has_cap'), 10, 3);

		$option =& $this->m_cache->get_option();
		if ($option->get_rewrite_feed_link())
			add_filter('feed_link', array(&$this, 'filter_feed_link'));
		if ($_GET[DiscloseSecret_POST_HTTP_AUTH] == DiscloseSecret_POST_HTTP_AUTH_DO
			&& $this->m_feed_privacy != DiscloseSecret_OPTION_FULL_CONTENT)
		{
			// Only filter RSS content if we're using HTTP auth
			add_filter('the_content', array(&$this, 'filter_the_content_rss'));
			add_filter('the_content_rss', array(&$this, 'filter_the_content_rss'));
			add_filter('the_excerpt_rss', array(&$this, 'filter_the_excerpt_rss'));
		}
	}

	// ----------------------------------------------------------------------------
	// save post filter
	// ----------------------------------------------------------------------------

	function clear_tag_attribute($reference, $post_key)
	{
		foreach ($reference as $id => $name)
			unset($_POST[$post_key. $id]);
	}

	function marshal_tag_attribute($reference, $post_key, $list)
	{
		$a = explode(',', $list);
		foreach ($a as $item)
		{
			$item = trim($item);
			foreach ($reference as $id => $ref)
			{
				if (!strcasecmp($item, $ref[DiscloseSecret_INDEX_ID])
					|| !strcasecmp($item, $ref[DiscloseSecret_INDEX_NAME]))
				{
					$_POST[$post_key. $id] = 'on';
				}
			}
		}
	}

	function tag_to_POST($match)
	{
		global $post;

		$name_index = 1;
		$value_index = 3;
		$pattern = "/[\s]*(.*?)[\s]*=[\s]*([\"\'])(.*?)\\2/is";
		preg_match_all($pattern, stripslashes(trim($match[3])), $elements);

		$this->m_save_as_private = true;
		$count = count($elements[0]);

		// resetting values before parsing
		$option =& $this->m_cache->get_option();
		$this->clear_tag_attribute($option->get_caps(), DiscloseSecret_POST_CAP);
		$this->clear_tag_attribute($option->get_roles(), DiscloseSecret_POST_ROLE);
		$this->clear_tag_attribute($option->get_users(), DiscloseSecret_POST_USER);

		for ($n = 0; $n < $count; ++$n)
		{
			if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_CAP
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_CAP_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_caps(),
					DiscloseSecret_POST_CAP, $elements[$value_index][$n]);
			}
			else if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_ROLE
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_ROLE_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_roles(),
					DiscloseSecret_POST_ROLE, $elements[$value_index][$n]);
			}
			else if (strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_USER
//				|| strtolower($elements[$name_index][$n]) == DiscloseSecret_XHTML_ATTRIBUTE_USER_ALT_1
			)
			{
				$this->marshal_tag_attribute($option->get_users(),
					DiscloseSecret_POST_USER, $elements[$value_index][$n]);
			}
		}
		return "";
	}

	// Called when a post is saved, evaluates post tags
	function filter_content_save_pre($content)
	{
		// extract possible in-post postrestriction settings to $_POST array
		// for later processing; last occurance of multiple tags
		// will overwrite previous tags
		$this->m_save_as_private = false;
		$pattern = "/<!--[\s]*(". DiscloseSecret_XHTML_TAG.
//			"|". DiscloseSecret_XHTML_TAG_ALT_1.
//			"|". DiscloseSecret_XHTML_TAG_ALT_2.
			")([\s]+(.*?))?[\s]*[\/]?[\s]*-->/is";
		$content = preg_replace_callback($pattern, array(&$this, tag_to_POST), $content);
		return $content;
	}

	// Called before post status is updated on save. Marks
	// post as private if a post restriction is set
	function filter_status_save_pre($post_status)
	{
		if ($post_status == 'publish')
		{
			// this is for in-post tags
			if ($this->m_disclosesecret_tag_error)
				return 'private';

			// this is for admin panel settings
			$option =& $this->m_cache->get_option();
			foreach ($option->get_caps() as $cap)
			{
				if ($_POST[DiscloseSecret_POST_CAP. $cap[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
			foreach ($option->get_roles() as $role)
			{
				if ($_POST[DiscloseSecret_POST_ROLE. $role[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
			foreach ($option->get_users() as $user)
			{
				if ($_POST[DiscloseSecret_POST_USER. $user[DiscloseSecret_INDEX_ID]])
					return 'private';
			}
		}
		return $post_status;
	}

	// Called when a post is saved, updates the post restriction
	function filter_post_save($post_ID)
	{
		$postmeta =& $this->m_cache->get_postmeta($post_ID);
		$postmeta->set_from_POST();
		$postmeta->save();
	}

	// ----------------------------------------------------------------------------
	// runtime filter
	// ----------------------------------------------------------------------------

	// Enable users with the right restriction to read a private post
	// This is required because single post viewing has special logic
	// $allcaps = Capabilities the user currently has
	// $caps = Primitive capabilities being tested / requested
	// $args = array with:
	// $args[0] = original meta capability requested
	// $args[1] = user being tested
	// $args[2] = post id to view
	// See code for assumptions
	function filter_user_has_cap($allcaps, $caps, $args)
	{
		// This handler is only set up to deal with the read_private_posts
		// capability. Ignore all other calls into here.
		if (!in_array('read_private_posts', $caps))
			return $allcaps;

		// As of WP 2.0, read_private_posts is only requested when viewing
		// a single post. When this happens, the args[] array has three values,
		// as shown above. If WP changes the args[], this plugin will break

		$postmeta =& $this->m_cache->get_postmeta($args[2]);
		$intersect_cap = array_intersect(array_keys($allcaps), $postmeta->get_caps());
		$has_cap = !empty($intersect_cap);

		$current_user = wp_get_current_user();
		$has_role = false;
		$has_user = false;
		if (!empty($current_user))
		{
			$intersect_role = array_intersect($current_user->roles, $postmeta->get_roles());
			$has_role = !empty($intersect_role);
			$has_user = in_array($current_user->id, $postmeta->get_users());
		}

		$rpp_caps = array();
		$rpp_caps['read_private_posts'] = ($has_cap || $has_role || $has_user);
		return $rpp_caps;
	}

	// Left join with the custom fields table so we can filter
	// based on restriction in the where clause
	function filter_posts_join($join, $posts = NULL)
	{
		// We add a left join here on the postmeta table --
		// this could conceivably break if another plugin
		// manipulates the join
		global $wpdb;

		if (!$posts)
			$posts = $wpdb->posts;

		$join .= " LEFT JOIN {$wpdb->postmeta} AS disclosesecret_meta ".
			"ON $posts.ID = disclosesecret_meta.post_id ".
				"AND disclosesecret_meta.meta_key IN ('". DiscloseSecret_META_CAP. "' ".
					", '". DiscloseSecret_META_ROLE. "' ".
					", '". DiscloseSecret_META_USER. "') ";
		return $join;
	}

	// Change where clause to accept published posts and private posts
	// with appropriate capability
	function filter_posts_where($where)
	{
		// This could break if another plugin manipulates the where
		// clause, specifically the post_status part
		global $wpdb, $current_user;

		$user_caps = "'". implode("', '", array_keys($current_user->allcaps)). "'";
		$user_roles = "'". implode("', '", $current_user->roles). "'";
		$user_id = $current_user->id;

		$where = preg_replace("/([\w-]*?\.)?post_status[\s]*=[\s]*([\"\'])publish\\2/is",
			'(\\1post_status = "publish" OR (\\1post_status = "private" '.
			"AND ((disclosesecret_meta.meta_key = '". DiscloseSecret_META_CAP. "' ".
					"AND disclosesecret_meta.meta_value IN ($user_caps) ) ".
				"OR (disclosesecret_meta.meta_key = '". DiscloseSecret_META_ROLE. "' ".
					"AND disclosesecret_meta.meta_value IN ($user_roles) ) ".
				"OR (disclosesecret_meta.meta_key = '". DiscloseSecret_META_USER. "' ".
					"AND disclosesecret_meta.meta_value IN ($user_id) )) ))",
			$where);
		return $where;
	}

	// ----------------------------------------------------------------------------
	// rss filter
	// ----------------------------------------------------------------------------

	function replace_feed_teaser_placeholder($content)
	{
		global $post;

		$search[] = DiscloseSecret_PLACEHOLDER_PERMALINK;
		$replace[] = get_permalink($post->ID);
		$content = str_replace($search, $replace, $content);
		return $content;
	}

	// Called when using HTTP auth -- changes the post content
	function filter_the_content_rss($content)
	{
		global $post;

		if ($post->post_status != 'private')
			return $content;

		$option =& $this->m_cache->get_option();
		switch ($option->get_feed_privacy())
		{
			case DiscloseSecret_OPTION_EXCERPT_ONLY:
				return apply_filters('the_excerpt_rss', get_the_excerpt(true));
		}
		return $this->replace_feed_teaser_placeholder($option->get_feed_teaser());
	}

	// Called when using HTTP auth -- changes the post excerpt
	function filter_the_excerpt_rss($excerpt)
	{
		global $post;

		if ($post->post_status != 'private')
			return $excerpt;

		$option =& $this->m_cache->get_option();
		switch ($option->get_feed_privacy())
		{
			case DiscloseSecret_OPTION_EXCERPT_ONLY:
				return $excerpt;
		}
		return $this->replace_feed_teaser_placeholder($option->get_feed_teaser());
	}

	// Rewrites RSS feed links to support http authentication
	// if the user is logged in
	function filter_feed_link($output)
	{
		$delim = (strpos($output, '?') === false) ? '?' : '&';
		return $output. $delim. DiscloseSecret_POST_HTTP_AUTH. '='. DiscloseSecret_POST_HTTP_AUTH_DO;
	}
}

?>